+{
   locale_version => 1.14,
   entry => <<'ENTRY', # for DUCET v7.0.0
010D      ; [.1952.0020.0002] # LATIN SMALL LETTER C WITH CARON
0063 030C ; [.1952.0020.0002] # LATIN SMALL LETTER C WITH CARON
010C      ; [.1952.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0043 030C ; [.1952.0020.0008] # LATIN CAPITAL LETTER C WITH CARON
0123      ; [.19DB.0020.0002] # LATIN SMALL LETTER G WITH CEDILLA
0067 0327 ; [.19DB.0020.0002] # LATIN SMALL LETTER G WITH CEDILLA
0122      ; [.19DB.0020.0008] # LATIN CAPITAL LETTER G WITH CEDILLA
0047 0327 ; [.19DB.0020.0008] # LATIN CAPITAL LETTER G WITH CEDILLA
0137      ; [.1A3A.0020.0002] # LATIN SMALL LETTER K WITH CEDILLA
006B 0327 ; [.1A3A.0020.0002] # LATIN SMALL LETTER K WITH CEDILLA
0136      ; [.1A3A.0020.0008] # LATIN CAPITAL LETTER K WITH CEDILLA
004B 0327 ; [.1A3A.0020.0008] # LATIN CAPITAL LETTER K WITH CEDILLA
013C      ; [.1A6D.0020.0002] # LATIN SMALL LETTER L WITH CEDILLA
006C 0327 ; [.1A6D.0020.0002] # LATIN SMALL LETTER L WITH CEDILLA
013B      ; [.1A6D.0020.0008] # LATIN CAPITAL LETTER L WITH CEDILLA
004C 0327 ; [.1A6D.0020.0008] # LATIN CAPITAL LETTER L WITH CEDILLA
0146      ; [.1AA0.0020.0002] # LATIN SMALL LETTER N WITH CEDILLA
006E 0327 ; [.1AA0.0020.0002] # LATIN SMALL LETTER N WITH CEDILLA
0145      ; [.1AA0.0020.0008] # LATIN CAPITAL LETTER N WITH CEDILLA
004E 0327 ; [.1AA0.0020.0008] # LATIN CAPITAL LETTER N WITH CEDILLA
0157      ; [.1B32.0020.0002] # LATIN SMALL LETTER R WITH CEDILLA
0072 0327 ; [.1B32.0020.0002] # LATIN SMALL LETTER R WITH CEDILLA
0156      ; [.1B32.0020.0008] # LATIN CAPITAL LETTER R WITH CEDILLA
0052 0327 ; [.1B32.0020.0008] # LATIN CAPITAL LETTER R WITH CEDILLA
0161      ; [.1B56.0020.0002] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.1B56.0020.0002] # LATIN SMALL LETTER S WITH CARON
0160      ; [.1B56.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.1B56.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
017E      ; [.1BFF.0020.0002] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.1BFF.0020.0002] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.1BFF.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.1BFF.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
ENTRY
};
